@@ -9,10 +9,10 @@ import spyctl.commands.create as c
 import spyctl.commands.diff as d
 import spyctl.commands.get as g
 import spyctl.commands.merge as m
+import spyctl.commands.update as u
 import spyctl.commands.validate as v
 import spyctl.config.configs as cfgs
 import spyctl.config.secrets as s
-import spyctl.commands.update as u
 import spyctl.spyctl_lib as lib
 from spyctl.commands.apply import handle_apply
 from spyctl.commands.delete import handle_delete
@@ -505,24 +505,55 @@ def delete(resource, name_or_id, yes=False):
 @click.option(
     "-f",
     "--filename",
-    help="Target file of the diff.",
+    help="Target file(s) of the diff.",
     metavar="",
-    required=True,
-    type=click.File(),
+    type=lib.FileList(),
+    cls=lib.MutuallyExclusiveEatAll,
+    mutually_exclusive=["policy"],
+)
+@click.option(
+    "-p",
+    "--policy",
+    is_flag=False,
+    flag_value="all",
+    default=None,
+    help="Target policy name(s) or uid(s) of the diff. If supplied with no"
+    " argument, set to 'all'.",
+    metavar="",
+    type=lib.ListParam(),
+    cls=lib.MutuallyExclusiveOption,
+    mutually_exclusive=["filename"],
 )
 @click.option(
     "-w",
     "--with-file",
-    help="File to diff with target file.",
+    "with_file",
+    help="File to diff with target.",
     metavar="",
     type=click.File(),
+    cls=lib.MutuallyExclusiveOption,
+    mutually_exclusive=["with_policy"],
+)
+@click.option(
+    "-P",
+    "--with-policy",
+    "with_policy",
+    help="Policy uid to diff with target. If supplied with no argument then"
+    " spyctl will attempt to find a policy matching the uid in the target's"
+    " metadata.",
+    metavar="",
+    is_flag=False,
+    flag_value="matching",
+    cls=lib.MutuallyExclusiveOption,
+    mutually_exclusive=["with_file"],
 )
 @click.option(
     "-l",
     "--latest",
     is_flag=True,
-    help="Diff file with latest records using the value of lastTimestamp in"
-    " metadata",
+    help=f"Diff target with latest records using the value of"
+    f" '{lib.LATEST_TIMESTAMP_FIELD}' in '{lib.METADATA_FIELD}'."
+    " This replaces --start-time.",
     metavar="",
 )
 @click.option(
@@ -530,7 +561,7 @@ def delete(resource, name_or_id, yes=False):
     "--start-time",
     "st",
     help="Start time of the query for fingerprints to diff."
-    " Only used if --latest and --with-file are not set."
+    " Only used if --latest, --with-file, --with-policy are not set."
     " Default is 24 hours ago.",
     default="24h",
     type=lib.time_inp,
@@ -540,14 +571,96 @@ def delete(resource, name_or_id, yes=False):
     "--end-time",
     "et",
     help="End time of the query for fingerprints to diff."
-    " Only used if --with-file is not set."
+    " Only used if --with-file, and --with-policy are not set."
     " Default is now.",
     default=time.time(),
     type=lib.time_inp,
 )
-def diff(filename, st, et, with_file=None, latest=False):
-    """Diff FingerprintsGroups with SpyderbatBaselines and SpyderbatPolicies"""
-    d.handle_diff(filename, with_file, st, et, latest)
+@click.option(
+    "-y",
+    "--yes",
+    "--assume-yes",
+    is_flag=True,
+    help='Automatic yes to prompts; assume "yes" as answer to all prompts and'
+    " run non-interactively.",
+)
+def diff(
+    filename,
+    policy,
+    st,
+    et,
+    yes=False,
+    with_file=None,
+    with_policy=None,
+    latest=False,
+):
+    """Diff target Baselines and Policies with other Resources.
+
+      Diff'ing in Spyctl requires a target Resource (e.g. a Baseline or Policy
+    document you are maintaining) and a Resource to diff with the target.
+    A target can be either a local file supplied using the -f option or a policy
+    you've applied to the Spyderbat Backend supplied with the -p option.
+    By default, target's are diff'd with relevant* Fingerprints from the last 24
+    hours to now. Targets may also be diff'd with local files with the -w option
+    or with data from an existing applied policy using the -P option.
+
+      The output of a diff shows you any lines that would be added to or removed
+    from your target Resource as a result of a Merge. diffs may also be performed
+    in bulk. Bulk diffs are outputted to a pager like 'less' or 'more'.
+
+      To maintain a target Resource effectively, the goal should be to get to
+    get to a point where the diff no longer displays added or removed lines (other
+    than timestamps).
+
+    \b
+    Examples:
+      # diff a local policy file with relevant* Fingerprints from the last
+      # 24hrs to now:
+      spyctl diff -f policy.yaml\n
+    \b
+      # diff a local policy file with relevant* Fingerprints from its
+      # latestTimestamp field to now:
+      spyctl diff -f policy.yaml --latest\n
+    \b
+      # diff an existing applied policy with relevant* Fingerprints from the
+      # last 24hrs to now:
+      spyctl diff -p <NAME_OR_UID>\n
+    \b
+      # Bulk diff all existing policies with relevant* Fingerprints from the
+      # last 24hrs to now:
+      spyctl diff -p\n
+    \b
+      # Bulk diff multiple policies with relevant* Fingerprints from the
+      # last 24hrs to now:
+      spyctl diff -p <NAME_OR_UID1>,<NAME_OR_UID2>\n
+    \b
+      # Bulk diff all files in cwd matching a pattern with relevant*
+      # Fingerprints from the last 24hrs to now:
+      spyctl diff -f *.yaml\n
+    \b
+      # diff an existing applied policy with a local file:
+      spyctl diff -p <NAME_OR_UID> --with-file fingerprints.yaml\n
+    \b
+      # diff a local file with data from an existing applied policy
+      spyctl diff -f policy.yaml -P <NAME_OR_UID>\n
+    \b
+      # diff a local file with a valid UID in its metadata with the matching
+      # policy in the Spyderbat Backend
+      spyctl diff -f policy.yaml -P
+
+    * Each policy has one or more Selectors in its spec field,
+    relevant Fingerprints are those that match those Selectors.
+
+    For time field options such as --start-time and --end-time you can
+    use (m) for minutes, (h) for hours (d) for days, and (w) for weeks back
+    from now or provide timestamps in epoch format.
+
+    Note: Long time ranges or "get" commands in a context consisting of
+    multiple machines can take a long time.
+    """  # noqa E501
+    if yes:
+        cli.set_yes_option()
+    d.handle_diff(filename, policy, with_file, with_policy, st, et, latest)
 
 
 # ----------------------------------------------------------------- #
@@ -581,6 +694,40 @@ class GetCommand(lib.ArgumentParametersCommand):
                 ),
             ],
         },
+        {
+            "resource": [lib.FINGERPRINTS_RESOURCE],
+            "args": [
+                click.option(
+                    "-f",
+                    "--filename",
+                    help="Input file to create filters from. Used if you want"
+                    " to get fingerprints related to another resource."
+                    " E.g. the Fingerprint Groups for a given Baseline file.",
+                    metavar="",
+                    type=click.File(),
+                ),
+                click.option(
+                    "-p",
+                    "--policy",
+                    help="Policy uid to build filters from. This will download"
+                    " a policy from the spyderbat backend, then filter"
+                    " Fingerprints based on the policy's selectors.",
+                    metavar="",
+                ),
+            ],
+        },
+        {
+            "resource": [lib.CONNECTIONS_RESOURCE],
+            "args": [
+                click.option(
+                    "--ignore-ips",
+                    "ignore_ips",
+                    is_flag=True,
+                    help="Ignores differing ips in the table output."
+                    " Off by default.",
+                ),
+            ],
+        },
     ]
 
 
@@ -648,22 +795,13 @@ class GetCommand(lib.ArgumentParametersCommand):
         lib.OUTPUT_CHOICES + [lib.OUTPUT_WIDE], case_sensitive=False
     ),
 )
-@click.option(
-    "-f",
-    "--filename",
-    help="Input file to create filters from. Used if you want to get resources"
-    " directly related to another resource. E.g. the Fingerprint Groups for a"
-    " given Baseline file.",
-    metavar="",
-    type=click.File(),
-)
 @click.option(
     "-l",
     "--latest",
     help="Starting time of the query is set to the"
-    f" {lib.LATEST_TIMESTAMP_FIELD}"
-    f" field the input resource's {lib.METADATA_FIELD}."
-    " [Requires '--filename' option to be set]",
+    f" '{lib.LATEST_TIMESTAMP_FIELD}'"
+    f" field the input resource's '{lib.METADATA_FIELD}' and replaces"
+    " --start-time. [Requires '--filename' option to be set]",
     is_flag=True,
 )
 @click.option(
@@ -703,12 +841,22 @@ def get(
 ):
     """Display one or many Spyderbat Resources.
 
+    See https://spyctl.readthedocs.io/en/latest/user/resources.html for a full
+    list of available resource.
+
+    \b
     Some resources are retrieved from from databases where a time range can
     be specified:
     - Fingerprints
     - Pods
     - Namespaces
+    - Nodes
+    - Processes
+    - Connections
+    - RedFlags
+    - OpsFlags
 
+    \b
     Other resources come from databases where time ranges are not applicable:
     - Clusters
     - Machines
@@ -718,19 +866,23 @@ def get(
     Examples:
       # Get all observed Pods for the last hour:
       spyctl get pods -t 1h\n
+    \b
       # Get all observed Pods from 4 hours ago to 2 hours ago
       spyctl get pods -t 4h -e 2h\n
+    \b
       # Get observed pods for a specific time range (using epoch timestamps)
       spyctl get pods -t 1675364629 -e 1675368229\n
-      # Get a Fingerprint Group of all runs of httpd.service for the
-      # last 24 hours and output to a yaml file
+    \b
+      # Get a Fingerprint Group of all runs of httpd.service for the last 24
+      # hours and output to a yaml file
       spyctl get fingerprints httpd.service -o yaml > fprints.yaml\n
+    \b
       # Get the latest fingerprints related to a policy yaml file
       spyctl get fingerprints -f policy.yaml --latest
 
     For time field options such as --start-time and --end-time you can
-    use (h) for hour (d) for days, and (w) for weeks back from now or
-    provide timestamps in epoch format.
+    use (m) for minutes, (h) for hours (d) for days, and (w) for weeks back
+    from now or provide timestamps in epoch format.
 
     Note: Long time ranges or "get" commands in a context consisting of
     multiple machines can take a long time.
@@ -761,24 +913,55 @@ def get(
 @click.option(
     "-f",
     "--filename",
-    help="Target file of the merge.",
+    help="Target file(s) of the merge.",
     metavar="",
-    required=True,
-    type=click.File(),
+    type=lib.FileList(),
+    cls=lib.MutuallyExclusiveEatAll,
+    mutually_exclusive=["policy"],
+)
+@click.option(
+    "-p",
+    "--policy",
+    is_flag=False,
+    flag_value=m.ALL,
+    default=None,
+    help="Target policy name(s) or uid(s) of the merge. If supplied with no"
+    " argument, set to 'all'.",
+    metavar="",
+    type=lib.ListParam(),
+    cls=lib.MutuallyExclusiveOption,
+    mutually_exclusive=["filename"],
 )
 @click.option(
     "-w",
     "--with-file",
-    help="File to merge into target file.",
+    "with_file",
+    help="File to merge into target.",
     metavar="",
     type=click.File(),
+    cls=lib.MutuallyExclusiveOption,
+    mutually_exclusive=["with_policy"],
+)
+@click.option(
+    "-P",
+    "--with-policy",
+    "with_policy",
+    help="Policy uid to merge with target. If supplied with no argument then"
+    " spyctl will attempt to find a policy matching the uid in the"
+    " target's metadata.",
+    metavar="",
+    is_flag=False,
+    flag_value=m.MATCHING,
+    cls=lib.MutuallyExclusiveOption,
+    mutually_exclusive=["with_file"],
 )
 @click.option(
     "-l",
     "--latest",
     is_flag=True,
     help=f"Merge file with latest records using the value of"
-    f" {lib.LATEST_TIMESTAMP_FIELD} in the input file's metadata",
+    f" '{lib.LATEST_TIMESTAMP_FIELD}' in the target's '{lib.METADATA_FIELD}'."
+    " This replaces --start-time.",
     metavar="",
 )
 @click.option(
@@ -792,7 +975,7 @@ def get(
     "--start-time",
     "st",
     help="Start time of the query for fingerprints to merge."
-    " Only used if --latest and --with-file are not set."
+    " Only used if --latest, --with-file, and --with-policy are not set."
     " Default is 24 hours ago.",
     default="24h",
     type=lib.time_inp,
@@ -802,18 +985,139 @@ def get(
     "--end-time",
     "et",
     help="End time of the query for fingerprints to merge."
-    " Only used if --with-file is not set."
+    " Only used if --with-file and --with-policy are not set."
     " Default is now.",
     default=time.time(),
     type=lib.time_inp,
 )
-def merge(filename, output, st, et, with_file=None, latest=False):
-    """Merge FingerprintsGroups into SpyderbatBaselines and
-    SpyderbatPolicies
-    """
+@click.option(
+    "-O",
+    "--output-to-file",
+    help="Should output merge to a file. Unique filename created from the name"
+    " in the object's metadata.",
+    is_flag=True,
+)
+@click.option(
+    "-y",
+    "--yes",
+    "--assume-yes",
+    is_flag=True,
+    help='Automatic yes to prompts; assume "yes" as answer to all prompts and'
+    " run non-interactively.",
+    cls=lib.MutuallyExclusiveOption,
+    mutually_exclusive=["yes_except"],
+)
+@click.option(
+    "-Y",
+    "--yes-except",
+    "--assume-yes-except-review",
+    is_flag=True,
+    help='Automatic yes to merge prompts; assume "yes" as answer to all merge'
+    " prompts but still prompts review of policy updates before applying.",
+    cls=lib.MutuallyExclusiveOption,
+    mutually_exclusive=["yes"],
+)
+def merge(
+    filename,
+    policy,
+    output,
+    st,
+    et,
+    yes=False,
+    yes_except=False,
+    with_file=None,
+    with_policy=None,
+    latest=False,
+    output_to_file=False,
+):
+    """Merge target Baselines and Policies with other Resources.
+
+      Merging in Spyctl requires a target Resource (e.g. a Baseline or Policy
+    document you are maintaining) and a Resource to merge into the target.
+    A target can either be a local file supplied using the -f option or a policy
+    you've applied to the Spyderbat Backend supplied with the -p option.
+    By default, target's are merged with relevant* Fingerprints from the last 24
+    hours to now. Targets may also be merged with local files with the -w option
+    or with data from an existing applied policy using the -P option.
+
+      When merging a single local file with another resource, the output will
+    be sent to stdout. WARNING: Do not redirect output to the same file you
+    used as input. You may use the -O flag to output the merged data to a
+    unique file with a name generate by Spyctl.
+
+      When bulk merging local files, the output for each merge operation will
+    be outputted to unique files generated by Spyctl (the same as supplying the
+    -O flag mentioned above).
+
+      When merging existing applied policies in bulk or individually, the default
+    destination for the output will be to apply it directly to the Spyderbat Backend (you
+    will have a chance to review the merge before any changes are applied).
+    This removes the requirement to deal with local files when managing policies. However,
+    it is a good idea to back up policies in a source-control repository. You can also
+    use the -O operation to send the output of this merge to a local file.
+
+    \b
+    Examples:
+      # merge a local policy file with relevant* Fingerprints from the last
+      # 24hrs to now:
+      spyctl merge -f policy.yaml\n
+    \b
+      # merge a local policy file with relevant* Fingerprints from its
+      # latestTimestamp field to now:
+      spyctl merge -f policy.yaml --latest\n
+    \b
+      # merge an existing applied policy with relevant* Fingerprints from the
+      # last 24hrs to now:
+      spyctl merge -p <NAME_OR_UID>\n
+    \b
+      # Bulk merge all existing policies with relevant* Fingerprints from the
+      # last 24hrs to now:
+      spyctl merge -p\n
+    \b
+      # Bulk merge multiple policies with relevant* Fingerprints from the
+      # last 24hrs to now:
+      spyctl merge -p <NAME_OR_UID1>,<NAME_OR_UID2>\n
+    \b
+      # Bulk merge all files in cwd matching a pattern with relevant*
+      # Fingerprints from the last 24hrs to now:
+      spyctl merge -f *.yaml\n
+    \b
+      # merge an existing applied policy with a local file:
+      spyctl merge -p <NAME_OR_UID> --with-file fingerprints.yaml\n
+    \b
+      # merge a local file with data from an existing applied policy
+      spyctl merge -f policy.yaml -P <NAME_OR_UID>\n
+    \b
+      # merge a local file with a valid UID in its metadata with the matching
+      # policy in the Spyderbat Backend
+      spyctl merge -f policy.yaml -P
+
+    * Each policy has one or more Selectors in its spec field,
+    relevant Fingerprints are those that match those Selectors.
+
+    For time field options such as --start-time and --end-time you can
+    use (m) for minutes, (h) for hours (d) for days, and (w) for weeks back
+    from now or provide timestamps in epoch format.
+
+    Note: Long time ranges or "get" commands in a context consisting of
+    multiple machines can take a long time.
+    """  # noqa E501
+    if yes or yes_except:
+        cli.set_yes_option()
     if output == lib.OUTPUT_DEFAULT:
         output = lib.OUTPUT_YAML
-    m.handle_merge(filename, with_file, st, et, latest, output)
+    m.handle_merge(
+        filename,
+        policy,
+        with_file,
+        with_policy,
+        st,
+        et,
+        latest,
+        output,
+        output_to_file,
+        yes_except,
+    )
 
 
 # ----------------------------------------------------------------- #
